import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import pystan
import seaborn as sb
from matplotlib import cm


df = pd.read_csv('MVData.csv')
data = df.values
Y = data.T[0]
X1 = data.T[1]
X2 = data.T[2]
for y, x1, x2 in zip(Y, X1, X2):
    print(y, x1, x2)
    
Z1 = (X1 - np.mean(X1)) / np.std(X1)
Z2 = (X2 - np.mean(X2)) / np.std(X2)
print('X1: mean = {0:.3f}   std = {1:.3f}'.format(np.mean(X1), np.std(X1)))
print('X2: mean = {0:.3f}   std = {1:.3f}'.format(np.mean(X2), np.std(X2)))

sm = pystan.StanModel(file = 'mv_regress.stan')
print('Compiling ende.')

Data = {'N':len(Y), 'Y':Y, 'X1':Z1, 'X2':Z2}
fit = sm.sampling(data = Data, n_jobs = 1)
print(fit)

b0_hat = np.median(fit['b0'])
b1_hat = np.median(fit['b1'])
b2_hat = np.median(fit['b2'])


sb.kdeplot(fit['b0'])
plt.title('b0: median = {0:.2f}'.format(b0_hat))
plt.savefig('Figb0_c.png')
plt.show()

sb.kdeplot(fit['b1'])
plt.title('b1: median = {0:.2f}'.format(b1_hat))
plt.savefig('Figb1_c.png')
plt.show()

sb.kdeplot(fit['b2'])
plt.title('b2: median = {0:.2f}'.format(b2_hat))
plt.savefig('Figb2_c.png')
plt.show()

fig = plt.figure()
ax = fig.add_subplot(projection='3d')

ax.scatter(Z1, Z2, Y, color='blue')

Xs = np.linspace(min(Z1), max(Z1), 100)
Ys = np.linspace(min(Z2), max(Z2), 100)
Xg, Yg = np.meshgrid(Xs, Ys)

Z = np.empty((100, 100))
for iy in range(100):
    for ix in range(100):
        Z[iy][ix] = b0_hat + b1_hat * Xs[ix] + b2_hat * Ys[iy]
    
ax.plot_surface(Xg, Yg, Z, cmap = cm.viridis, alpha = 0.3)
ax.set_xlabel('Z1')
ax.set_ylabel('Z2')
ax.set_zlabel('Y')
ax.set_title('Multivariate Regression')

for i in range(len(Y)):
    plt.plot([Z1[i],Z1[i]], [Z2[i],Z2[i]],
             [b0_hat + b1_hat*Z1[i] + b2_hat*Z2[i], Y[i]], c = 'm')
plt.plot([0], [0], [b0_hat], 'D', color = 'r', 
         label = (r'(0,0,$\hat{b}_0}$)') + '\n' + (r'$\hatb_0$={0:.2f}'.format(b0_hat)))
plt.legend()
plt.title('You can rotate this graph by dragging')
plt.savefig('FigRegress_c.png')
plt.show()
